@@ -110,7 +110,7 @@ file_list_ready_cb (GList *files,
 	GdkPixbuf *pixbuf;
 	GtkWidget *label;
 	GString *str;
-	PangoFontDescription *desc;
+	PangoAttrList *attr_list;
 
 	dialog = GTK_DIALOG (fcd);
 	details = fcd->details;
@@ -201,14 +201,15 @@ file_list_ready_cb (GList *files,
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (details->titles_vbox),
 			    label, FALSE, FALSE, 0);
-	gtk_widget_modify_font (label, NULL);
-	desc = pango_font_description_new ();
-	pango_font_description_set_weight (desc, PANGO_WEIGHT_BOLD);
-	pango_font_description_set_size (desc,
-		pango_font_description_get_size (gtk_widget_get_style (label)->font_desc) * PANGO_SCALE_LARGE);
-	gtk_widget_modify_font (label, desc);
-	pango_font_description_free (desc);
-	gtk_widget_show (label);
+
+	attr_list = pango_attr_list_new ();
+	pango_attr_list_insert (attr_list, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
+	pango_attr_list_insert (attr_list, pango_attr_scale_new (PANGO_SCALE_LARGE));
+	g_object_set (label,
+		      "attributes", attr_list,
+		      NULL);
+
+	pango_attr_list_unref (attr_list);
 
 	label = gtk_label_new (secondary_text);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
