@@ -30,7 +30,6 @@
 #include "fm-desktop-icon-view.h"
 #include "fm-error-reporting.h"
 #include <stdlib.h>
-#include <eel/eel-background.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-gtk-macros.h>
@@ -42,7 +41,7 @@
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
-#include <libnautilus-private/nautilus-directory-background.h>
+#include <libnautilus-private/nautilus-clipboard-monitor.h>
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
@@ -108,6 +107,8 @@ struct FMIconViewDetails
 	int num_screens;
 
 	gboolean compact;
+
+	gulong clipboard_handler_id;
 };
 
 
@@ -145,11 +146,11 @@ static const SortCriterion sort_criteria[] = {
 		N_("Keep icons sorted by modification date in rows")
 	},
 	{
-		NAUTILUS_FILE_SORT_BY_EMBLEMS,
-		"emblems",
-		"Sort by Emblems",
-		N_("by _Emblems"),
-		N_("Keep icons sorted by emblems in rows")
+		NAUTILUS_FILE_SORT_BY_TRASHED_TIME,
+		"trashed",
+		"Sort by Trash Time",
+		N_("by T_rash Time"),
+		N_("Keep icons sorted by trash time in rows")
 	}
 };
 
@@ -166,7 +167,8 @@ static void                 fm_icon_view_update_click_mode            (FMIconVie
 static void                 fm_icon_view_set_directory_tighter_layout (FMIconView           *icon_view,
 								       NautilusFile         *file,
 								       gboolean              tighter_layout);
-static gboolean              fm_icon_view_supports_manual_layout      (FMIconView           *icon_view);
+static gboolean             fm_icon_view_supports_manual_layout       (FMIconView           *icon_view);
+static gboolean             fm_icon_view_supports_scaling	      (FMIconView           *icon_view);
 static void                 fm_icon_view_reveal_selection             (FMDirectoryView      *view);
 static const SortCriterion *get_sort_criterion_by_sort_type           (NautilusFileSortType  sort_type);
 static void                 set_sort_criterion_by_sort_type           (FMIconView           *icon_view,
@@ -178,7 +180,14 @@ static void                 preview_audio                             (FMIconVie
 								       NautilusFile         *file,
 								       gboolean              start_flag);
 static void                 update_layout_menus                       (FMIconView           *view);
-
+static NautilusFileSortType get_default_sort_order                    (NautilusFile         *file,
+								       gboolean             *reversed);
+static void default_sort_order_changed_callback            (gpointer callback_data);
+static void default_sort_in_reverse_order_changed_callback (gpointer callback_data);
+static void default_use_tighter_layout_changed_callback    (gpointer callback_data);
+static void default_zoom_level_changed_callback            (gpointer callback_data);
+static void labels_beside_icons_changed_callback           (gpointer callback_data);
+static void all_columns_same_width_changed_callback	   (gpointer callback_data);
 
 static void fm_icon_view_iface_init (NautilusViewIface *iface);
 
@@ -187,7 +196,7 @@ G_DEFINE_TYPE_WITH_CODE (FMIconView, fm_icon_view, FM_TYPE_DIRECTORY_VIEW,
 						fm_icon_view_iface_init));
 
 static void
-fm_icon_view_destroy (GtkObject *object)
+fm_icon_view_destroy (GtkWidget *object)
 {
 	FMIconView *icon_view;
 
@@ -198,6 +207,12 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->react_to_icon_change_idle_id = 0;
         }
 
+	if (icon_view->details->clipboard_handler_id != 0) {
+		g_signal_handler_disconnect (nautilus_clipboard_monitor_get (),
+					     icon_view->details->clipboard_handler_id);
+		icon_view->details->clipboard_handler_id = 0;
+	}
+
 	/* kill any sound preview process that is ongoing */
 	preview_audio (icon_view, NULL, FALSE);
 
@@ -206,7 +221,7 @@ fm_icon_view_destroy (GtkObject *object)
 		icon_view->details->icons_not_positioned = NULL;
 	}
 
-	GTK_OBJECT_CLASS (fm_icon_view_parent_class)->destroy (object);
+	GTK_WIDGET_CLASS (fm_icon_view_parent_class)->destroy (object);
 }
 
 
@@ -219,13 +234,35 @@ fm_icon_view_finalize (GObject *object)
 
 	g_free (icon_view->details);
 
+	g_signal_handlers_disconnect_by_func (nautilus_preferences,
+					      default_sort_order_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_preferences,
+					      default_sort_in_reverse_order_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_icon_view_preferences,
+					      default_use_tighter_layout_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_icon_view_preferences,
+					      default_zoom_level_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_icon_view_preferences,
+					      labels_beside_icons_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_compact_view_preferences,
+					      default_zoom_level_changed_callback,
+					      icon_view);
+	g_signal_handlers_disconnect_by_func (nautilus_compact_view_preferences,
+					      all_columns_same_width_changed_callback,
+					      icon_view);
+
 	G_OBJECT_CLASS (fm_icon_view_parent_class)->finalize (object);
 }
 
 static NautilusIconContainer *
 get_icon_container (FMIconView *icon_view)
 {
-	return NAUTILUS_ICON_CONTAINER (GTK_BIN (icon_view)->child);
+	return NAUTILUS_ICON_CONTAINER (gtk_bin_get_child (GTK_BIN (icon_view)));
 }
 
 static gboolean
@@ -257,40 +294,69 @@ get_stored_icon_position_callback (NautilusIconContainer *container,
 
 	/* If it is the desktop directory, maybe the gnome-libs metadata has information about it */
 
-	/* Get the scale of the icon from the metadata. */
-	scale_string = nautilus_file_get_metadata
-		(file, NAUTILUS_METADATA_KEY_ICON_SCALE, "1");
-	position->scale = g_ascii_strtod (scale_string, NULL);
-	if (errno != 0) {
- 		position->scale = 1.0;
- 	}
+	/* Disable scaling if not on the desktop */
+	if (fm_icon_view_supports_scaling (icon_view)) {
+		/* Get the scale of the icon from the metadata. */
+		scale_string = nautilus_file_get_metadata
+			(file, NAUTILUS_METADATA_KEY_ICON_SCALE, "1");
+		position->scale = g_ascii_strtod (scale_string, NULL);
+		if (errno != 0) {
+			position->scale = 1.0;
+		}
 
-	g_free (scale_string);
+		g_free (scale_string);
+	} else {
+		position->scale = 1.0;
+	}
 	
 	return position_good;
 }
 
-
-static gboolean
-set_sort_criterion (FMIconView *icon_view, const SortCriterion *sort)
+static void
+real_set_sort_criterion (FMIconView *icon_view,
+                         const SortCriterion *sort,
+                         gboolean clear)
 {
-	if (sort == NULL) {
-		return FALSE;
+	NautilusFile *file;
+
+	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (icon_view));
+
+	if (clear) {
+		nautilus_file_set_metadata (file,
+					    NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_BY, NULL, NULL);
+		nautilus_file_set_metadata (file,
+					    NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED, NULL, NULL);
+		icon_view->details->sort =
+			get_sort_criterion_by_sort_type	(get_default_sort_order
+							 (file, &icon_view->details->sort_reversed));
+	} else {
+		/* Store the new sort setting. */
+		fm_icon_view_set_directory_sort_by (icon_view,
+						    file,
+						    sort->metadata_text);
 	}
-	if (icon_view->details->sort == sort) {
-		return FALSE;
-	}
-	icon_view->details->sort = sort;
 
-	/* Store the new sort setting. */
-	fm_icon_view_set_directory_sort_by (icon_view,
-					    fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (icon_view)),
-					    sort->metadata_text);
-	
 	/* Update the layout menus to match the new sort setting. */
 	update_layout_menus (icon_view);
+}
 
-	return TRUE;
+static void
+set_sort_criterion (FMIconView *icon_view, const SortCriterion *sort)
+{
+	if (sort == NULL ||
+	    icon_view->details->sort == sort) {
+		return;
+	}
+
+	icon_view->details->sort = sort;
+
+        real_set_sort_criterion (icon_view, sort, FALSE);
+}
+
+static void
+clear_sort_criterion (FMIconView *icon_view)
+{
+	real_set_sort_criterion (icon_view, NULL, TRUE);
 }
 
 static void
@@ -575,6 +641,16 @@ fm_icon_view_supports_auto_layout (FMIconView *view)
 		 supports_auto_layout, (view));
 }
 
+static gboolean
+fm_icon_view_supports_scaling (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return EEL_CALL_METHOD_WITH_RETURN_VALUE
+		(FM_ICON_VIEW_CLASS, view,
+		 supports_scaling, (view));
+}
+
 static gboolean
 fm_icon_view_supports_manual_layout (FMIconView *view)
 {
@@ -617,12 +693,14 @@ update_layout_menus (FMIconView *view)
 	gboolean is_auto_layout;
 	GtkAction *action;
 	const char *action_name;
+	NautilusFile *file;
 
 	if (view->details->icon_action_group == NULL) {
 		return;
 	}
 
 	is_auto_layout = fm_icon_view_using_auto_layout (view);
+	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
 
 	if (fm_icon_view_supports_auto_layout (view)) {
 		/* Mark sort criterion. */
@@ -643,6 +721,15 @@ update_layout_menus (FMIconView *view)
 		gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 					      view->details->sort_reversed);
 		gtk_action_set_sensitive (action, is_auto_layout);
+
+		action = gtk_action_group_get_action (view->details->icon_action_group,
+		                                      FM_ACTION_SORT_TRASH_TIME);
+
+		if (file != NULL && nautilus_file_is_in_trash (file)) {
+			gtk_action_set_visible (action, TRUE);
+		} else {
+			gtk_action_set_visible (action, FALSE);
+		}
 	}
 
 	action = gtk_action_group_get_action (view->details->icon_action_group,
@@ -655,6 +742,10 @@ update_layout_menus (FMIconView *view)
 					      FM_ACTION_CLEAN_UP);
 	gtk_action_set_sensitive (action, !is_auto_layout);	
 
+	if (FM_IS_DESKTOP_ICON_VIEW (view)) {
+		gtk_action_set_label (action, _("_Organize Desktop by Name"));
+	}
+
 	action = gtk_action_group_get_action (view->details->icon_action_group,
 					      FM_ACTION_KEEP_ALIGNED);
 	gtk_action_set_visible (action,
@@ -681,17 +772,35 @@ fm_icon_view_get_directory_sort_by (FMIconView *icon_view,
 static NautilusFileSortType default_sort_order = NAUTILUS_FILE_SORT_BY_DISPLAY_NAME;
 
 static NautilusFileSortType
-get_default_sort_order (void)
+get_default_sort_order (NautilusFile *file, gboolean *reversed)
 {
 	static gboolean auto_storaged_added = FALSE;
+	NautilusFileSortType retval;
 
 	if (auto_storaged_added == FALSE) {
 		auto_storaged_added = TRUE;
-		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
-					       (int *) &default_sort_order);
+		eel_g_settings_add_auto_enum (nautilus_preferences,
+					      NAUTILUS_PREFERENCES_DEFAULT_SORT_ORDER,
+					      (int *) &default_sort_order);
+		eel_g_settings_add_auto_boolean (nautilus_preferences,
+						 NAUTILUS_PREFERENCES_DEFAULT_SORT_IN_REVERSE_ORDER,
+						 &default_sort_in_reverse_order);
+
 	}
 
-	return CLAMP (default_sort_order, NAUTILUS_FILE_SORT_BY_DISPLAY_NAME, NAUTILUS_FILE_SORT_BY_EMBLEMS);
+	retval = nautilus_file_get_default_sort_type (file, reversed);
+
+	if (retval == NAUTILUS_FILE_SORT_NONE) {
+
+		if (reversed != NULL) {
+			*reversed = default_sort_in_reverse_order;
+		}
+
+		retval = CLAMP (default_sort_order, NAUTILUS_FILE_SORT_BY_DISPLAY_NAME,
+				NAUTILUS_FILE_SORT_BY_MTIME);
+	}
+
+	return retval;
 }
 
 static char *
@@ -699,7 +808,7 @@ fm_icon_view_real_get_directory_sort_by (FMIconView *icon_view,
 					 NautilusFile *file)
 {
 	const SortCriterion *default_sort_criterion;
-	default_sort_criterion = get_sort_criterion_by_sort_type (get_default_sort_order ());
+	default_sort_criterion = get_sort_criterion_by_sort_type (get_default_sort_order (file, NULL));
 	g_return_val_if_fail (default_sort_criterion != NULL, NULL);
 
 	return nautilus_file_get_metadata
@@ -726,7 +835,7 @@ fm_icon_view_real_set_directory_sort_by (FMIconView *icon_view,
 					 const char *sort_by)
 {
 	const SortCriterion *default_sort_criterion;
-	default_sort_criterion = get_sort_criterion_by_sort_type (get_default_sort_order ());
+	default_sort_criterion = get_sort_criterion_by_sort_type (get_default_sort_order (file, NULL));
 	g_return_if_fail (default_sort_criterion != NULL);
 
 	nautilus_file_set_metadata
@@ -748,28 +857,17 @@ fm_icon_view_get_directory_sort_reversed (FMIconView *icon_view,
 		 get_directory_sort_reversed, (icon_view, file));
 }
 
-static gboolean
-get_default_sort_in_reverse_order (void)
-{
-	static gboolean auto_storaged_added = FALSE;
-	
-	if (auto_storaged_added == FALSE) {
-		auto_storaged_added = TRUE;
-		eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
-						  &default_sort_in_reverse_order);
-	}
-
-	return default_sort_in_reverse_order;
-}
-
 static gboolean
 fm_icon_view_real_get_directory_sort_reversed (FMIconView *icon_view,
 					       NautilusFile *file)
 {
+	gboolean reversed;
+
+	get_default_sort_order (file, &reversed);
 	return nautilus_file_get_boolean_metadata
 		(file,
 		 NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
-		 get_default_sort_in_reverse_order ());
+		 reversed);
 }
 
 static void
@@ -791,11 +889,13 @@ fm_icon_view_real_set_directory_sort_reversed (FMIconView *icon_view,
 					       NautilusFile *file,
 					       gboolean sort_reversed)
 {
+	gboolean reversed;
+
+	get_default_sort_order (file, &reversed);
 	nautilus_file_set_boolean_metadata
 		(file,
 		 NAUTILUS_METADATA_KEY_ICON_VIEW_SORT_REVERSED,
-		 get_default_sort_in_reverse_order (),
-		 sort_reversed);
+		 reversed, sort_reversed);
 }
 
 static gboolean
@@ -833,23 +933,6 @@ fm_icon_view_set_directory_keep_aligned (FMIconView *icon_view,
 		 keep_aligned);
 }
 
-/* maintainence of auto layout boolean */
-static gboolean default_directory_manual_layout = FALSE;
-
-static gboolean
-get_default_directory_manual_layout (void)
-{
-	static gboolean auto_storaged_added = FALSE;
-	
-	if (auto_storaged_added == FALSE) {
-		auto_storaged_added = TRUE;
-		eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_MANUAL_LAYOUT,
-						       &default_directory_manual_layout);
-	}
-
-	return default_directory_manual_layout;
-}
-
 static gboolean
 fm_icon_view_get_directory_auto_layout (FMIconView *icon_view,
 					NautilusFile *file)
@@ -871,10 +954,8 @@ static gboolean
 fm_icon_view_real_get_directory_auto_layout (FMIconView *icon_view,
 					     NautilusFile *file)
 {
-
-
 	return nautilus_file_get_boolean_metadata
-		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT, !get_default_directory_manual_layout ());
+		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT, TRUE);
 }
 
 static void
@@ -902,7 +983,7 @@ fm_icon_view_real_set_directory_auto_layout (FMIconView *icon_view,
 
 	nautilus_file_set_boolean_metadata
 		(file, NAUTILUS_METADATA_KEY_ICON_VIEW_AUTO_LAYOUT,
-		 !get_default_directory_manual_layout (),
+		 TRUE,
 		 auto_layout);
 }
 /* maintainence of tighter layout boolean */
@@ -922,11 +1003,12 @@ static gboolean
 get_default_directory_tighter_layout (void)
 {
 	static gboolean auto_storaged_added = FALSE;
-	
+
 	if (auto_storaged_added == FALSE) {
 		auto_storaged_added = TRUE;
-		eel_preferences_add_auto_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
-						       &default_directory_tighter_layout);
+		eel_g_settings_add_auto_boolean (nautilus_icon_view_preferences,
+						 NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
+						 &default_directory_tighter_layout);
 	}
 
 	return default_directory_tighter_layout;
@@ -978,6 +1060,14 @@ real_supports_auto_layout (FMIconView *view)
 	return TRUE;
 }
 
+static gboolean
+real_supports_scaling (FMIconView *view)
+{
+	g_return_val_if_fail (FM_IS_ICON_VIEW (view), FALSE);
+
+	return FALSE;
+}
+
 static gboolean
 real_supports_manual_layout (FMIconView *view)
 {
@@ -1059,10 +1149,12 @@ get_default_zoom_level (FMIconView *icon_view)
 
 	if (!auto_storage_added) {
 		auto_storage_added = TRUE;
-		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-					       (int *) &default_zoom_level);
-		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
-					       (int *) &default_compact_zoom_level);
+		eel_g_settings_add_auto_enum (nautilus_icon_view_preferences,
+					      NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+					      (int *) &default_zoom_level);
+		eel_g_settings_add_auto_enum (nautilus_compact_view_preferences,
+					      NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
+					      (int *) &default_compact_zoom_level);
 	}
 
 	return CLAMP (DEFAULT_ZOOM_LEVEL(icon_view), NAUTILUS_ZOOM_LEVEL_SMALLEST, NAUTILUS_ZOOM_LEVEL_LARGEST);
@@ -1075,15 +1167,16 @@ set_labels_beside_icons (FMIconView *icon_view)
 
 	if (fm_icon_view_supports_labels_beside_icons (icon_view)) {
 		labels_beside = fm_icon_view_is_compact (icon_view) ||
-				eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
-		
+			g_settings_get_boolean (nautilus_icon_view_preferences,
+						NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS);
+
 		if (labels_beside) {
 			nautilus_icon_container_set_label_position
-				(get_icon_container (icon_view), 
+				(get_icon_container (icon_view),
 				 NAUTILUS_ICON_LABEL_POSITION_BESIDE);
 		} else {
 			nautilus_icon_container_set_label_position
-				(get_icon_container (icon_view), 
+				(get_icon_container (icon_view),
 				 NAUTILUS_ICON_LABEL_POSITION_UNDER);
 		}
 	}
@@ -1095,7 +1188,8 @@ set_columns_same_width (FMIconView *icon_view)
 	gboolean all_columns_same_width;
 
 	if (fm_icon_view_is_compact (icon_view)) {
-		all_columns_same_width = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH);
+		all_columns_same_width = g_settings_get_boolean (nautilus_compact_view_preferences,
+								 NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH);
 		nautilus_icon_container_set_all_columns_same_width (get_icon_container (icon_view), all_columns_same_width);
 	}
 }
@@ -1108,7 +1202,7 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	NautilusFile *file;
 	int level;
 	char *sort_name;
-	
+
 	g_return_if_fail (FM_IS_ICON_VIEW (view));
 
 	icon_view = FM_ICON_VIEW (view);
@@ -1123,27 +1217,6 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	/* kill any sound preview process that is ongoing */
 	preview_audio (icon_view, NULL, FALSE);
 
-	/* FIXME bugzilla.gnome.org 45060: Should use methods instead
-	 * of hardcoding desktop knowledge in here.
-	 */
-	if (FM_IS_DESKTOP_ICON_VIEW (view)) {
-		nautilus_connect_desktop_background_to_file_metadata (NAUTILUS_ICON_CONTAINER (icon_container), file);
-	} else {
-		GdkDragAction default_action;
-		
-		if (nautilus_window_info_get_window_type (fm_directory_view_get_nautilus_window (view)) == NAUTILUS_WINDOW_NAVIGATION) {
-			default_action = NAUTILUS_DND_ACTION_SET_AS_GLOBAL_BACKGROUND;
-		} else {
-			default_action = NAUTILUS_DND_ACTION_SET_AS_FOLDER_BACKGROUND;
-		}
-		
-		nautilus_connect_background_to_file_metadata 
-			(icon_container, 
-			 file, 
-			 default_action);
-	}
-
-	
 	/* Set up the zoom level from the metadata. */
 	if (fm_directory_view_supports_zooming (FM_DIRECTORY_VIEW (icon_view))) {
 		if (icon_view->details->compact) {
@@ -1194,17 +1267,40 @@ fm_icon_view_begin_loading (FMDirectoryView *view)
 	update_layout_menus (icon_view);
 }
 
+static void
+icon_view_notify_clipboard_info (NautilusClipboardMonitor *monitor,
+                                 NautilusClipboardInfo *info,
+                                 FMIconView *icon_view)
+{
+	GList *icon_data;
+
+	icon_data = NULL;
+	if (info && info->cut) {
+		icon_data = info->files;
+	}
+
+	nautilus_icon_container_set_highlighted_for_clipboard (
+		get_icon_container (icon_view), icon_data);
+}
+
 static void
 fm_icon_view_end_loading (FMDirectoryView *view,
 			  gboolean all_files_seen)
 {
 	FMIconView *icon_view;
 	GtkWidget *icon_container;
+	NautilusClipboardMonitor *monitor;
+	NautilusClipboardInfo *info;
 
 	icon_view = FM_ICON_VIEW (view);
 
 	icon_container = GTK_WIDGET (get_icon_container (icon_view));
 	nautilus_icon_container_end_loading (NAUTILUS_ICON_CONTAINER (icon_container), all_files_seen);
+
+	monitor = nautilus_clipboard_monitor_get ();
+	info = nautilus_clipboard_monitor_get_clipboard_info (monitor);
+
+	icon_view_notify_clipboard_info (monitor, info, icon_view);
 }
 
 static NautilusZoomLevel
@@ -1316,14 +1412,6 @@ fm_icon_view_can_zoom_out (FMDirectoryView *view)
 		> NAUTILUS_ZOOM_LEVEL_SMALLEST;
 }
 
-static GtkWidget * 
-fm_icon_view_get_background_widget (FMDirectoryView *view) 
-{
-	g_return_val_if_fail (FM_IS_ICON_VIEW (view), NULL);
-
-	return GTK_WIDGET (get_icon_container (FM_ICON_VIEW (view)));
-}
-
 static gboolean
 fm_icon_view_is_empty (FMDirectoryView *view)
 {
@@ -1502,7 +1590,7 @@ static const GtkActionEntry icon_view_entries[] = {
   /* tooltip */                  N_("Restore each selected icon to its original size"),
                                  G_CALLBACK (action_unstretch_callback) },
   /* name, stock id */         { "Clean Up", NULL,
-  /* label, accelerator */       N_("Clean _Up by Name"), NULL,
+  /* label, accelerator */       N_("_Organize by Name"), NULL,
   /* tooltip */                  N_("Reposition icons to better fit in the window and avoid overlapping"),
                                  G_CALLBACK (action_clean_up_callback) },
 };
@@ -1546,10 +1634,10 @@ static const GtkRadioActionEntry arrange_radio_entries[] = {
     N_("By Modification _Date"), NULL,
     N_("Keep icons sorted by modification date in rows"),
     NAUTILUS_FILE_SORT_BY_MTIME },
-  { "Sort by Emblems", NULL,
-    N_("By _Emblems"), NULL,
-    N_("Keep icons sorted by emblems in rows"),
-    NAUTILUS_FILE_SORT_BY_EMBLEMS },
+  { "Sort by Trash Time", NULL,
+    N_("By T_rash Time"), NULL,
+    N_("Keep icons sorted by trash time in rows"),
+    NAUTILUS_FILE_SORT_BY_TRASHED_TIME },
 };
 
 static void
@@ -1601,7 +1689,7 @@ fm_icon_view_merge_menus (FMDirectoryView *view)
 		gtk_action_set_visible (action, FALSE);
 	}
 
-	if (FM_IS_DESKTOP_ICON_VIEW (icon_view)) {
+	if (fm_icon_view_supports_scaling (icon_view)) {
 		gtk_ui_manager_add_ui (ui_manager,
 				       icon_view->details->icon_merge_id,
 				       POPUP_PATH_ICON_APPEARANCE,
@@ -1664,6 +1752,9 @@ fm_icon_view_update_menus (FMDirectoryView *view)
 				  && icon_container != NULL
 				  && !nautilus_icon_container_has_stretch_handles (icon_container));
 
+	gtk_action_set_visible (action,
+				fm_icon_view_supports_scaling (icon_view));
+
 	action = gtk_action_group_get_action (icon_view->details->icon_action_group,
 					      FM_ACTION_UNSTRETCH);
 	g_object_set (action, "label",
@@ -1675,6 +1766,9 @@ fm_icon_view_update_menus (FMDirectoryView *view)
 				  icon_container != NULL
 				  && nautilus_icon_container_is_stretched (icon_container));
 
+	gtk_action_set_visible (action,
+				fm_icon_view_supports_scaling (icon_view));
+
 	nautilus_file_list_free (selection);
 
 	editable = fm_directory_view_is_editable (view);
@@ -1692,8 +1786,7 @@ fm_icon_view_reset_to_defaults (FMDirectoryView *view)
 	icon_view = FM_ICON_VIEW (view);
 	icon_container = get_icon_container (icon_view);
 
-	set_sort_criterion (icon_view, get_sort_criterion_by_sort_type (get_default_sort_order ()));
-	set_sort_reversed (icon_view, get_default_sort_in_reverse_order ());
+	clear_sort_criterion (icon_view);
 	nautilus_icon_container_set_keep_aligned 
 		(icon_container, get_default_directory_keep_aligned ());
 	nautilus_icon_container_set_tighter_layout
@@ -1701,14 +1794,6 @@ fm_icon_view_reset_to_defaults (FMDirectoryView *view)
 
 	nautilus_icon_container_sort (icon_container);
 
-	/* Switch to manual layout of the default calls for it.
-	 * This needs to happen last for the sort order menus
-	 * to be in sync.
-	 */
- 	if (get_default_directory_manual_layout ()) {
- 		switch_to_manual_layout (icon_view);
- 	}
-
 	update_layout_menus (icon_view);
 
 	fm_icon_view_restore_default_zoom_level (view);
@@ -1780,6 +1865,14 @@ fm_icon_view_using_manual_layout (FMDirectoryView *view)
 	return !fm_icon_view_using_auto_layout (FM_ICON_VIEW (view));
 }
 
+static void
+fm_icon_view_set_is_active (FMDirectoryView *view,
+			    gboolean is_active)
+{
+	nautilus_icon_container_set_active (get_icon_container (FM_ICON_VIEW (view)),
+					    is_active);
+}
+
 static void
 fm_icon_view_widget_to_file_operation_position (FMDirectoryView *view,
 						GdkPoint *position)
@@ -1934,6 +2027,8 @@ play_file (gpointer callback_data)
 	char **argv;
 	GError *error;
 	char *uri;
+	GFile *gfile;
+	char *path;
 
 	icon_view = FM_ICON_VIEW (callback_data);
 	
@@ -1941,7 +2036,21 @@ play_file (gpointer callback_data)
 	icon_view->details->audio_preview_timeout = 0;
 
 	file = icon_view->details->audio_preview_file;
-	uri = nautilus_file_get_uri (file);
+	gfile = nautilus_file_get_location (file);
+	path = g_file_get_path (gfile);
+
+	/* if we have a local path, use that instead of the native URI.
+	 * this can be useful for special GVfs mounts, such as cdda://
+	 */
+	if (path) {
+		uri = g_filename_to_uri (path, NULL, NULL);
+	} else {
+		uri = nautilus_file_get_uri (file);
+	}
+
+	g_object_unref (gfile);
+	g_free (path);
+
 	argv = get_preview_argv (uri);
 	g_free (uri);
 	if (argv == NULL) {
@@ -2401,8 +2510,8 @@ static void
 fm_icon_view_emblems_changed (FMDirectoryView *directory_view)
 {
 	g_assert (FM_IS_ICON_VIEW (directory_view));
-	
-	nautilus_icon_container_request_update_all (get_icon_container (FM_ICON_VIEW (directory_view)));	
+
+	nautilus_icon_container_request_update_all (get_icon_container (FM_ICON_VIEW (directory_view)));
 }
 
 static void
@@ -2469,28 +2578,6 @@ default_use_tighter_layout_changed_callback (gpointer callback_data)
 	nautilus_icon_container_request_update_all (icon_container);
 }
 
-static void
-default_use_manual_layout_changed_callback (gpointer callback_data)
-{
-	FMIconView *icon_view;
-	NautilusFile *file;
-	NautilusIconContainer *icon_container;
-
-	g_return_if_fail (FM_IS_ICON_VIEW (callback_data));
-
-	icon_view = FM_ICON_VIEW (callback_data);
-
-	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (icon_view));
-	icon_container = get_icon_container (icon_view);
-	g_return_if_fail (NAUTILUS_IS_ICON_CONTAINER (icon_container));
-
-	nautilus_icon_container_set_auto_layout (
-		icon_container,
-		fm_icon_view_get_directory_auto_layout (icon_view, file));
-
-	nautilus_icon_container_request_update_all (icon_container);
-}
-
 static void
 default_zoom_level_changed_callback (gpointer callback_data)
 {
@@ -2556,8 +2643,6 @@ fm_icon_view_sort_directories_first_changed (FMDirectoryView *directory_view)
 	}
 }
 
-/* GtkObject methods. */
-
 static gboolean
 icon_view_can_accept_item (NautilusIconContainer *container,
 			   NautilusFile *target_item,
@@ -2599,7 +2684,7 @@ fm_icon_view_update_click_mode (FMIconView *icon_view)
 	icon_container = get_icon_container (icon_view);
 	g_assert (icon_container != NULL);
 
-	click_mode = eel_preferences_get_enum (NAUTILUS_PREFERENCES_CLICK_POLICY);
+	click_mode = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_CLICK_POLICY);
 
 	nautilus_icon_container_set_single_click_mode (icon_container,
 						       click_mode == NAUTILUS_CLICK_POLICY_SINGLE);
@@ -2681,7 +2766,7 @@ create_icon_container (FMIconView *icon_view)
 
 	icon_container = fm_icon_container_new (icon_view);
 
-	GTK_WIDGET_SET_FLAGS (icon_container, GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (GTK_WIDGET (icon_container), TRUE);
 	
 	g_signal_connect_object (icon_container, "focus_in_event",
 			 G_CALLBACK (focus_in_event_callback), icon_view, 0);
@@ -2860,8 +2945,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	G_OBJECT_CLASS (klass)->set_property = fm_icon_view_set_property;
 	G_OBJECT_CLASS (klass)->finalize = fm_icon_view_finalize;
 
-	GTK_OBJECT_CLASS (klass)->destroy = fm_icon_view_destroy;
-	
+	GTK_WIDGET_CLASS (klass)->destroy = fm_icon_view_destroy;
 	GTK_WIDGET_CLASS (klass)->screen_changed = fm_icon_view_screen_changed;
 	GTK_WIDGET_CLASS (klass)->scroll_event = fm_icon_view_scroll_event;
 	
@@ -2875,7 +2959,6 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	fm_directory_view_class->clear = fm_icon_view_clear;
 	fm_directory_view_class->end_loading = fm_icon_view_end_loading;
 	fm_directory_view_class->file_changed = fm_icon_view_file_changed;
-	fm_directory_view_class->get_background_widget = fm_icon_view_get_background_widget;
 	fm_directory_view_class->get_selected_icon_locations = fm_icon_view_get_selected_icon_locations;
 	fm_directory_view_class->get_selection = fm_icon_view_get_selection;
 	fm_directory_view_class->get_selection_for_file_transfer = fm_icon_view_get_selection;
@@ -2886,6 +2969,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 	fm_directory_view_class->restore_default_zoom_level = fm_icon_view_restore_default_zoom_level;
 	fm_directory_view_class->reveal_selection = fm_icon_view_reveal_selection;
 	fm_directory_view_class->select_all = fm_icon_view_select_all;
+	fm_directory_view_class->set_is_active = fm_icon_view_set_is_active;
 	fm_directory_view_class->set_selection = fm_icon_view_set_selection;
 	fm_directory_view_class->invert_selection = fm_icon_view_invert_selection;
 	fm_directory_view_class->compare_files = compare_files;
@@ -2906,6 +2990,7 @@ fm_icon_view_class_init (FMIconViewClass *klass)
 
 	klass->clean_up = fm_icon_view_real_clean_up;
 	klass->supports_auto_layout = real_supports_auto_layout;
+	klass->supports_scaling = real_supports_scaling;
 	klass->supports_manual_layout = real_supports_manual_layout;
 	klass->supports_keep_aligned = real_supports_keep_aligned;
 	klass->supports_labels_beside_icons = real_supports_labels_beside_icons;
@@ -2947,7 +3032,7 @@ static void
 fm_icon_view_iface_init (NautilusViewIface *iface)
 {
 	fm_directory_view_init_view_iface (iface);
-	
+
 	iface->get_view_id = fm_icon_view_get_id;
 	iface->get_first_visible_file = icon_view_get_first_visible_file;
 	iface->scroll_to_file = icon_view_scroll_to_file;
@@ -2960,7 +3045,7 @@ fm_icon_view_init (FMIconView *icon_view)
 	static gboolean setup_sound_preview = FALSE;
 	NautilusIconContainer *icon_container;
 
-        g_return_if_fail (GTK_BIN (icon_view)->child == NULL);
+        g_return_if_fail (gtk_bin_get_child (GTK_BIN (icon_view)) == NULL);
 
 	icon_view->details = g_new0 (FMIconViewDetails, 1);
 	icon_view->details->sort = &sort_criteria[0];
@@ -2975,37 +3060,42 @@ fm_icon_view_init (FMIconView *icon_view)
 						 NAUTILUS_ICON_LAYOUT_L_R_T_B);
 
 	if (!setup_sound_preview) {
-		eel_preferences_add_auto_enum (NAUTILUS_PREFERENCES_PREVIEW_SOUND,
-					       &preview_sound_auto_value);
-		
+		eel_g_settings_add_auto_enum (nautilus_preferences,
+					      NAUTILUS_PREFERENCES_PREVIEW_SOUND,
+					      &preview_sound_auto_value);
+
 		setup_sound_preview = TRUE;
 	}
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_ORDER,
-						  default_sort_order_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER,
-						  default_sort_in_reverse_order_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
-						  default_use_tighter_layout_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_MANUAL_LAYOUT,
-						  default_use_manual_layout_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
-						  default_zoom_level_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS,
-						  labels_beside_icons_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
+	g_signal_connect_swapped (nautilus_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_DEFAULT_SORT_ORDER,
+				  G_CALLBACK (default_sort_order_changed_callback),
+				  icon_view);
+	g_signal_connect_swapped (nautilus_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_DEFAULT_SORT_IN_REVERSE_ORDER,
+				  G_CALLBACK (default_sort_in_reverse_order_changed_callback),
+				  icon_view);
+	g_signal_connect_swapped (nautilus_icon_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_USE_TIGHTER_LAYOUT,
+				  G_CALLBACK (default_use_tighter_layout_changed_callback),
+				  icon_view);
+	g_signal_connect_swapped (nautilus_icon_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_ICON_VIEW_DEFAULT_ZOOM_LEVEL,
+				  G_CALLBACK (default_zoom_level_changed_callback),
+				  icon_view);
+	g_signal_connect_swapped (nautilus_icon_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_ICON_VIEW_LABELS_BESIDE_ICONS,
+				  G_CALLBACK (labels_beside_icons_changed_callback),
+				  icon_view);
 
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
-						  default_zoom_level_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
-	eel_preferences_add_callback_while_alive (NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH,
-						  all_columns_same_width_changed_callback,
-						  icon_view, G_OBJECT (icon_view));
+	g_signal_connect_swapped (nautilus_compact_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_COMPACT_VIEW_DEFAULT_ZOOM_LEVEL,
+				  G_CALLBACK (default_zoom_level_changed_callback),
+				  icon_view);
+	g_signal_connect_swapped (nautilus_compact_view_preferences,
+				  "changed::" NAUTILUS_PREFERENCES_COMPACT_VIEW_ALL_COLUMNS_SAME_WIDTH,
+				  G_CALLBACK (all_columns_same_width_changed_callback),
+				  icon_view);
 
 	g_signal_connect_object (get_icon_container (icon_view), "handle_netscape_url",
 				 G_CALLBACK (icon_view_handle_netscape_url), icon_view, 0);
@@ -3015,6 +3105,11 @@ fm_icon_view_init (FMIconView *icon_view)
 				 G_CALLBACK (icon_view_handle_text), icon_view, 0);
 	g_signal_connect_object (get_icon_container (icon_view), "handle_raw",
 				 G_CALLBACK (icon_view_handle_raw), icon_view, 0);
+
+	icon_view->details->clipboard_handler_id =
+		g_signal_connect (nautilus_clipboard_monitor_get (),
+		                  "clipboard_info",
+		                  G_CALLBACK (icon_view_notify_clipboard_info), icon_view);
 }
 
 static NautilusView *
