@@ -46,8 +46,6 @@
 #include <libnautilus-private/nautilus-clipboard-monitor.h>
 #include <libnautilus-private/nautilus-column-chooser.h>
 #include <libnautilus-private/nautilus-column-utilities.h>
-#include <libnautilus-private/nautilus-debug-log.h>
-#include <libnautilus-private/nautilus-directory-background.h>
 #include <libnautilus-private/nautilus-dnd.h>
 #include <libnautilus-private/nautilus-file-dnd.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
@@ -61,6 +59,9 @@
 #include <libnautilus-private/nautilus-clipboard.h>
 #include <libnautilus-private/nautilus-cell-renderer-text-ellipsized.h>
 
+#define DEBUG_FLAG NAUTILUS_DEBUG_LIST_VIEW
+#include <libnautilus-private/nautilus-debug.h>
+
 struct FMListViewDetails {
 	GtkTreeView *tree_view;
 	FMListModel *model;
@@ -90,12 +91,10 @@ struct FMListViewDetails {
 	int drag_y;
 
 	gboolean drag_started;
-
 	gboolean ignore_button_release;
-
 	gboolean row_selected_on_button_down;
-
 	gboolean menus_ready;
+	gboolean active;
 	
 	GHashTable *columns;
 	GtkWidget *column_editor;
@@ -857,10 +856,7 @@ row_expanded_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *pa
 		char *uri;
 
 		uri = nautilus_directory_get_uri (directory);
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "list view row expanded window=%p: %s",
-				    fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)),
-				    uri);
+		DEBUG ("Row expaded callback for uri %s", uri);
 		g_free (uri);
 
 		fm_directory_view_add_subdirectory (FM_DIRECTORY_VIEW (view), directory);
@@ -944,10 +940,7 @@ row_collapsed_callback (GtkTreeView *treeview, GtkTreeIter *iter, GtkTreePath *p
 	
 
 	uri = nautilus_file_get_uri (file);
-	nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-			    "list view row collapsed window=%p: %s",
-			    fm_directory_view_get_containing_window (FM_DIRECTORY_VIEW (view)),
-			    uri);
+	DEBUG ("Row collapsed callback for uri %s", uri);
 	g_free (uri);
 
 	unload_data = g_new (struct UnloadDelayData, 1);
@@ -1470,6 +1463,48 @@ filename_cell_data_func (GtkTreeViewColumn *column,
 	g_free (text);
 }
 
+static void
+setup_background (FMListView *view)
+{
+	GtkWidget *widget;
+	GdkRGBA color;
+	GtkStyleContext *style;
+	GdkWindow *window;
+	gboolean is_active = view->details->active;
+
+	widget = GTK_WIDGET (fm_list_view_get_tree_view (FM_LIST_VIEW (view)));
+
+	if (!gtk_widget_get_realized (widget)) {
+		return;
+	}
+
+	DEBUG ("Setting up background; is active %d", is_active);
+
+	style = gtk_widget_get_style_context (widget);
+	window = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget));
+
+	if (!is_active) {
+		gtk_style_context_get_background_color (style, GTK_STATE_FLAG_ACTIVE,
+							&color);
+		eel_make_color_inactive (&color);
+
+		gtk_widget_override_background_color (widget, GTK_STATE_FLAG_ACTIVE, &color);
+		gtk_style_context_set_background (style, window);
+	} else {
+		gtk_widget_override_background_color (widget, GTK_STATE_FLAG_ACTIVE, NULL);
+		gtk_style_context_set_background (style, window);
+	}
+}
+
+static void
+realize_event_callback (GtkWidget *tree_view,
+			gpointer user_data)
+{
+	FMListView *view = user_data;
+
+	setup_background (view);
+}
+
 static gboolean
 focus_in_event_callback (GtkWidget *widget, GdkEventFocus *event, gpointer user_data)
 {
@@ -1559,6 +1594,8 @@ create_and_set_up_tree_view (FMListView *view)
 	
     	g_signal_connect_object (view->details->tree_view, "focus_in_event",
 				 G_CALLBACK(focus_in_event_callback), view, 0);
+	g_signal_connect (view->details->tree_view, "realize",
+			  G_CALLBACK (realize_event_callback), view);
     
 	view->details->model = g_object_new (FM_TYPE_LIST_MODEL, NULL);
 	gtk_tree_view_set_model (view->details->tree_view, GTK_TREE_MODEL (view->details->model));
@@ -1945,12 +1982,6 @@ fm_list_view_file_changed (FMDirectoryView *view, NautilusFile *file, NautilusDi
 	}
 }
 
-static GtkWidget *
-fm_list_view_get_background_widget (FMDirectoryView *view)
-{
-	return GTK_WIDGET (view);
-}
-
 static void
 fm_list_view_get_selection_foreach_func (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
 {
@@ -2143,7 +2174,7 @@ fm_list_view_set_selection (FMDirectoryView *view, GList *selection)
 			gtk_tree_selection_select_iter (tree_selection,
 							(GtkTreeIter *)l->data);
 		}
-		eel_g_list_free_deep (iters);
+		g_list_free_full (iters, g_free);
 	}
 
 	g_signal_handlers_unblock_by_func (tree_selection, list_selection_changed_callback, view);
@@ -2178,7 +2209,7 @@ fm_list_view_invert_selection (FMDirectoryView *view)
 			gtk_tree_selection_unselect_iter (tree_selection,
 							(GtkTreeIter *)l->data);
 		}
-		eel_g_list_free_deep (iters);
+		g_list_free_full (iters, g_free);
 	}
 
 	g_list_free (selection);
@@ -3005,22 +3036,9 @@ static void
 real_set_is_active (FMDirectoryView *view,
 		    gboolean is_active)
 {
-	GtkWidget *tree_view;
-	GtkStyle *style;
-	GdkColor color;
+	FM_LIST_VIEW (view)->details->active = is_active;
 
-	tree_view = GTK_WIDGET (fm_list_view_get_tree_view (FM_LIST_VIEW (view)));
-
-	if (is_active) {
-		gtk_widget_modify_base (tree_view, GTK_STATE_NORMAL, NULL);
-	} else {
-		style = gtk_widget_get_style (tree_view);
-		color = style->base[GTK_STATE_INSENSITIVE];
-		gtk_widget_modify_base (tree_view, GTK_STATE_NORMAL, &color);
-	}
-
-	EEL_CALL_PARENT (FM_DIRECTORY_VIEW_CLASS,
-			 set_is_active, (view, is_active));
+	setup_background (FM_LIST_VIEW (view));
 }
 
 static void
@@ -3042,7 +3060,6 @@ fm_list_view_class_init (FMListViewClass *class)
         fm_directory_view_class->click_policy_changed = fm_list_view_click_policy_changed;
 	fm_directory_view_class->clear = fm_list_view_clear;
 	fm_directory_view_class->file_changed = fm_list_view_file_changed;
-	fm_directory_view_class->get_background_widget = fm_list_view_get_background_widget;
 	fm_directory_view_class->get_selection = fm_list_view_get_selection;
 	fm_directory_view_class->get_selection_for_file_transfer = fm_list_view_get_selection_for_file_transfer;
 	fm_directory_view_class->get_item_count = fm_list_view_get_item_count;
